@@ -351,7 +351,7 @@ class LightSetWaveform(Message):
         self.color = payload["color"]
         self.period = payload["period"]
         self.cycles = payload["cycles"]
-        self.duty_cycle = payload["duty_cycle"]
+        self.skew_ratio = payload["skew_ratio"]
         self.waveform = payload["waveform"]
         super(LightSetWaveform, self).__init__(MSG_IDS[LightSetWaveform], target_addr, source_id, seq_num, ack_requested, response_requested)
 
@@ -361,9 +361,41 @@ class LightSetWaveform(Message):
         color = b"".join(little_endian(bitstring.pack("uint:16", field)) for field in self.color)
         period = little_endian(bitstring.pack("uint:32", self.period))
         cycles = little_endian(bitstring.pack("float:32", self.cycles))
-        duty_cycle = little_endian(bitstring.pack("int:16", self.duty_cycle))
+        skew_ratio = little_endian(bitstring.pack("int:16", self.skew_ratio))
         waveform = little_endian(bitstring.pack("uint:8", self.waveform))
-        payload = reserved_8 + transient + color + period + cycles + duty_cycle + waveform
+        payload = reserved_8 + transient + color + period + cycles + skew_ratio + waveform
+
+        payloadUi = " ".join("{:02x}".format(c) for c in payload)
+        return payload
+
+
+class LightSetWaveformOptional(Message):
+    def __init__(self, target_addr, source_id, seq_num, payload, ack_requested=False, response_requested=False):
+        self.transient = payload["transient"]
+        self.color = payload["color"]
+        self.period = payload["period"]
+        self.cycles = payload["cycles"]
+        self.skew_ratio = payload["skew_ratio"]
+        self.waveform = payload["waveform"]
+        self.set_hue = payload["set_hue"]
+        self.set_saturation = payload["set_saturation"]
+        self.set_brightness = payload["set_brightness"]
+        self.set_kelvin = payload["set_kelvin"]
+        super(LightSetWaveformOptional, self).__init__(MSG_IDS[LightSetWaveformOptional], target_addr, source_id, seq_num, ack_requested, response_requested)
+
+    def get_payload(self):
+        reserved_8 = little_endian(bitstring.pack("uint:8", self.reserved))
+        transient = little_endian(bitstring.pack("uint:8", self.transient))
+        color = b"".join(little_endian(bitstring.pack("uint:16", field)) for field in self.color)
+        period = little_endian(bitstring.pack("uint:32", self.period))
+        cycles = little_endian(bitstring.pack("float:32", self.cycles))
+        skew_ratio = little_endian(bitstring.pack("int:16", self.skew_ratio))
+        waveform = little_endian(bitstring.pack("uint:8", self.waveform))
+        set_hue = little_endian(bitstring.pack("uint:8", self.set_hue))
+        set_saturation = little_endian(bitstring.pack("uint:8", self.set_saturation))
+        set_brightness = little_endian(bitstring.pack("uint:8", self.set_brightness))
+        set_kelvin = little_endian(bitstring.pack("uint:8", self.set_kelvin))
+        payload = reserved_8 + transient + color + period + cycles + skew_ratio + waveform + set_hue + set_saturation + set_brightness + set_kelvin
 
         payloadUi = " ".join("{:02x}".format(c) for c in payload)
         return payload
@@ -426,7 +458,7 @@ class LightStatePower(Message):
 
 ##### INFRARED MESSAGES #####
 
-class LightGetInfrared(Message): 
+class LightGetInfrared(Message):
     def __init__(self, target_addr, source_id, seq_num, payload={}, ack_requested=False, response_requested=False):
         super(LightGetInfrared, self).__init__(MSG_IDS[LightGetInfrared], target_addr, source_id, seq_num, ack_requested, response_requested)
 
@@ -556,9 +588,10 @@ MSG_IDS = {     GetService: 2,
                 LightGetPower: 116,
                 LightSetPower: 117,
                 LightStatePower: 118,
+                LightSetWaveformOptional: 119,
                 LightGetInfrared: 120,
                 LightStateInfrared: 121,
-                LightSetInfrared: 122, 
+                LightSetInfrared: 122,
                 MultiZoneSetColorZones: 501,
                 MultiZoneGetColorZones: 502,
                 MultiZoneStateZone: 503,
